'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.request = request;
exports.requestTransaction = requestTransaction;
exports.requestCursor = requestCursor;
exports.mapCursor = mapCursor;

/**
 * Transform IDBRequest to `Promise`,
 * which resolves on `success` or on `complete` when `tr` passed.
 *
 * @param {IDBRequest|IDBOpenDBRequest} req
 * @param {IDBTransaction} [tr]
 * @return {Promise}
 */

function request(req) {
  var tr = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  var result = undefined;
  return new Promise(function (resolve, reject) {
    req.onerror = handleError(reject);
    req.onsuccess = function (e) {
      result = e.target.result;
      if (!tr) resolve(e.target.result);
    };
    if (tr) tr.oncomplete = function () {
      return resolve(result);
    };
  });
}

/**
 * Transform `tr` to `Promise`.
 *
 * @param {IDBTransaction} tr
 * @return {Promise}
 */

function requestTransaction(tr) {
  return new Promise(function (resolve, reject) {
    tr.onerror = handleError(reject);
    tr.oncomplete = function () {
      return resolve();
    };
  });
}

/**
 * Call `iterator` for each `onsuccess` event.
 *
 * @param {IDBRequest} req
 * @param {Function} iterator
 * @return {Promise}
 */

function requestCursor(req, iterator) {
  // patch iterator to fix:
  // https://github.com/axemclion/IndexedDBShim/issues/204

  var keys = {}; // count unique keys
  var patchedIterator = function patchedIterator(cursor, stop) {
    if ((cursor.direction === 'prevunique' || cursor.direction === 'nextunique') && !cursor.source.multiEntry) {
      if (!keys[cursor.key]) {
        keys[cursor.key] = true;
        iterator(cursor, stop);
      } else {
        cursor.continue();
      }
    } else {
      iterator(cursor, stop);
    }
  };

  return new Promise(function (resolve, reject) {
    var stop = function stop() {
      return resolve();
    };
    req.onerror = handleError(reject);
    req.onsuccess = function (e) {
      var cursor = e.target.result;
      if (cursor) {
        patchedIterator(cursor, stop);
      } else {
        resolve();
      }
    };
  });
}

/**
 * Special helper to map values over cursor.
 *
 * @param {IDBRequest} req
 * @param {Function} iterator
 * @return {Promise}
 */

function mapCursor(req, iterator) {
  var result = [];
  return requestCursor(req, function (cursor, stop) {
    return iterator(cursor, result, stop);
  }).then(function () {
    return result;
  });
}

/**
 * Helper to handle errors and call `reject`.
 *
 * @param {Function} reject - from Promise constructor
 * @return {Function}
 */

function handleError(reject) {
  return function (e) {
    // prevent global error throw https://bugzilla.mozilla.org/show_bug.cgi?id=872873
    if (typeof e.preventDefault === 'function') e.preventDefault();
    reject(e.target.error);
  };
}